import { EditorView, basicSetup } from 'codemirror';
import { javascript } from '@codemirror/lang-javascript';
import { json } from '@codemirror/lang-json';
import { html } from '@codemirror/lang-html';
import { markdown } from '@codemirror/lang-markdown';
import { css } from '@codemirror/lang-css';
import { formatKeymap } from './extensions/tab';
let editor: EditorView = null;

type CreateOpts = {
  jsx?: boolean;
  typescript?: boolean;
  type?: 'javascript' | 'json' | 'html' | 'markdown' | 'css';
  hasBasicSetup?: boolean;
  extensions?: any[];
  hasKeymap?: boolean;
};
/**
 * 创建单例
 * @param el
 * @returns
 */
const createEditorInstance = (el?: HTMLDivElement, opts?: CreateOpts) => {
  if (editor && el) {
    el.appendChild(editor.dom);
    return editor;
  } else if (editor) {
    return editor;
  }
  const { type = 'javascript' } = opts || {};
  const extensions = opts?.extensions || [];
  const hasBaseicSetup = opts?.hasBasicSetup ?? true;
  const hasKeymap = opts?.hasKeymap ?? true;
  if (hasBaseicSetup) {
    extensions.unshift(basicSetup);
  }
  if (hasKeymap) {
    extensions.push(formatKeymap);
  }
  switch (type) {
    case 'json':
      extensions.push(json());
      break;
    case 'javascript':
      extensions.push(javascript({ jsx: opts?.jsx, typescript: opts?.typescript }));
      break;
    case 'css':
      extensions.push(css());
      break;
    case 'html':
      extensions.push(html());
      break;
    case 'markdown':
      extensions.push(markdown());
      break;
  }
  editor = new EditorView({
    extensions: extensions,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};

/**
 * 每次都创建新的实例
 * @param el
 * @returns
 */
export const createEditor = (el: HTMLDivElement, opts?: CreateOpts) => {
  const { type = 'javascript' } = opts || {};
  const extensions = opts?.extensions || [];
  const hasBaseicSetup = opts?.hasBasicSetup ?? true;
  const hasKeymap = opts?.hasKeymap ?? true;
  if (hasBaseicSetup) {
    extensions.unshift(basicSetup);
  }
  if (hasKeymap) {
    extensions.push(formatKeymap);
  }
  switch (type) {
    case 'json':
      extensions.push(json());
      break;
    case 'javascript':
      extensions.push(javascript({ jsx: opts?.jsx, typescript: opts?.typescript }));
      break;
    case 'css':
      extensions.push(css());
      break;
    case 'html':
      extensions.push(html());
      break;
    case 'markdown':
      extensions.push(markdown());
      break;
  }
  const editor = new EditorView({
    extensions: extensions,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};

export { editor, createEditorInstance };
